from yta_general_utils.programming.enum import YTAEnum as Enum


class MusicEngine(Enum):
    DEFAULT = 'default'
    """
    When this option is provided, the system will
    choose one of the available enum elements.
    """
    YOUTUBE = 'youtube'
    """
    The music will be generated by downloading it
    from our specific Youtube channel.
    """